-- Mod: FS22_RealTimeSync

RealTimeSync = {}

local RealTimeSync_mt = Class(RealTimeSync)

function RealTimeSync:new(mission, modDirectory, modName, logger)
	local self = setmetatable({}, RealTimeSync_mt)

	self.myName            = "RealTimeSync"
	self.logger            = logger
	self.isServer          = mission:getIsServer()
	self.isClient          = mission:getIsClient()
	self.isMPGame          = g_currentMission.missionDynamicInfo.isMultiplayer
	self.mission           = mission
	self.modDirectory      = modDirectory
	self.modName           = modName
	self.gameInfoDisplay   = mission.hud.gameInfoDisplay
  self.justSetFalse      = false

  source(modDirectory .. 'scripts/lib/fs22ModPrefSaver.lua')

  -- Load mod default settings
  self.settings = FS22PrefSaver:new(
    "FS22_RealTimeSync",
    "RealTimeSyncSettings.xml",
    true,
    {
      enableTimeSync      = false,
      serverOffset        = {12, "int"},
      timeFixHour         = {1, "int"},
      autoSetTime         = false,
      setValueTextSize    = {12, "int"},
      dayStartTimeSpring  = {6, "int"},
      dayEndTimeSpring    = {6, "int"},
      dayStartTimeSummer  = {5, "int"},
      dayEndTimeSummer    = {7, "int"},
      dayStartTimeAutumn  = {6, "int"},
      dayEndTimeAutumn    = {6, "int"},
      dayStartTimeWinter  = {7, "int"},
      dayEndTimeWinter    = {5, "int"},
    },
		function ()
			self.inspectText.size = self.gameInfoDisplay:scalePixelToScreenHeight(self.settings:getValue("setValueTextSize"))
		end,
		nil,
		self.logger
  )

	self.debugTimerRuns         = 0
	self.setValueTimerFrequency = 60
  self.inspectText            = {}
  self.menuTextSizes          = { 8, 10, 12, 14, 16 }

	local modDesc       = loadXMLFile("modDesc", modDirectory .. "modDesc.xml");
	self.version        = getXMLString(modDesc, "modDesc.version");
	delete(modDesc)

  self.logger:print(":new() Initialized", FS22Log.LOG_LEVEL.VERBOSE, "method_track")

  self.ServerUpdate = false
  self.ServerHour = 0
  self.ServerMinCleanUTC = 0
  self.TimeSpeed = 1

  return self

end

function RealTimeSync:save()
	self.settings:saveSettings()
end

function RealTimeSync:onStartMission(mission)
	-- Load the mod, make the box that info lives in.

	self.logger:print(JTSUtil.qConcat("Loaded - version : ", self.version), FS22Log.LOG_LEVEL.INFO, "user_info")

	if not self.isClient then
		return
	end

	-- Just call both, load fails gracefully if it doesn't exists.
	self.settings:loadSettings()
	self.settings:saveSettings()

	self.logger:print(":onStartMission()", FS22Log.LOG_LEVEL.VERBOSE, "method_track")

  RealTimeSync:updateDayLightHours(self)

end

function RealTimeSync:delete()
	-- clean up on remove
end

function RealTimeSync.addMenuOption(original, target, id, i18n_title, i18n_tooltip, options, callback)
	local menuOption = original:clone()

	menuOption.target = target
	menuOption.id     = id

	menuOption:setCallback("onClickCallback", callback)
	menuOption:setDisabled(false)

	local settingTitle = menuOption.elements[4]
	local toolTip      = menuOption.elements[6]

	menuOption:setTexts({unpack(options)})
	settingTitle:setText(g_i18n:getText(i18n_title))
	toolTip:setText(g_i18n:getText(i18n_tooltip))

	return menuOption
end

function RealTimeSync.initGui(self)

  g_realTimeSync.settings:loadSettings()

  local onOffMenuOptions = {
    "enableTimeSync"
  }

	local boolMenuOptions = {
		"autoSetTime"
	}

	if not g_realTimeSync.createdGUI then
		-- Create controls -- Skip if we've already done this once
		g_realTimeSync.createdGUI = true

		self.menuOption_ServerOffset = self.checkAutoMotorStart:clone()

		local title = TextElement.new()
		title:applyProfile("settingsMenuSubtitle", true)
		title:setText(g_i18n:getText("title_realTimeSync"))
		self.boxLayout:addElement(title)

		for _, thisOptionName in ipairs(onOffMenuOptions) do
			-- on off style options
			local thisFullOptName = "menuOption_" .. thisOptionName
			self[thisFullOptName] = RealTimeSync.addMenuOption(
				self.checkAutoMotorStart,
				g_realTimeSync,
				"realTimeSync_" .. thisOptionName,
				"setting_realTimeSync_" .. thisOptionName,
				"toolTip_realTimeSync_" .. thisOptionName,
				{
					g_i18n:getText("ui_off"),
          g_i18n:getText("ui_on")
				},
				"onMenuOptionChanged_onOffOpt"
			)
			self.boxLayout:addElement(self[thisFullOptName])
		end

    self.menuOption_ServerOffset = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_serverOffset",
        "setting_realTimeSync_serverOffset",
        "toolTip_realTimeSync_serverOffset",
        {
            "UTC-11:00", "UTC-10:00", "UTC-09:00", "UTC-08:00", "UTC-07:00", "UTC-06:00", "UTC-05:00", 
            "UTC-04:00", "UTC-03:00", "UTC-02:00", "UTC-01:00", "UTC","UTC+01:00", "UTC+02:00", "UTC+03:00", 
            "UTC+04:00", "UTC+05:00", "UTC+06:00", "UTC+07:00", "UTC+08:00", "UTC+09:00", "UTC+10:00", "UTC+11:00"
        },
        "onMenuOptionChanged_serverOffset"
    )

		self.boxLayout:addElement(self.menuOption_ServerOffset)

    -- Add button for Time Fix Hour
    self.menuOption_TimeFixHour = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_timeFixHour",
        "setting_realTimeSync_timeFixHour",
        "toolTip_realTimeSync_timeFixHour",
        {
            "Always",
            "00:00", "01:00", "02:00", "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00","11:00",
            "12:00", "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00", "21:00", "22:00","23:00"
        },
        "onMenuOptionChanged_timeFixHour"
    )
		self.boxLayout:addElement(self.menuOption_TimeFixHour)

		for _, thisOptionName in ipairs(boolMenuOptions) do
			-- Boolean style options
			local thisFullOptName = "menuOption_" .. thisOptionName
			self[thisFullOptName] = RealTimeSync.addMenuOption(
				self.checkAutoMotorStart,
				g_realTimeSync,
				"realTimeSync_" .. thisOptionName,
				"setting_realTimeSync_" .. thisOptionName,
				"toolTip_realTimeSync_" .. thisOptionName,
				{
					g_i18n:getText("ui_no"),
					g_i18n:getText("ui_yes")
				},
				"onMenuOptionChanged_boolOpt"
			)
			self.boxLayout:addElement(self[thisFullOptName])
		end

		local title = TextElement.new()
		title:applyProfile("settingsMenuSubtitle", true)
		title:setText(g_i18n:getText("title_realTimeSyncSeasons"))
		self.boxLayout:addElement(title)

    -- Add button for dayStartTimeSpring 
    self.menuOption_dayStartTimeSpring = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayStartTimeSpring",
        "setting_realTimeSync_dayStartTimeSpring",
        "toolTip_realTimeSync_dayStartTimeSpring",
        {
            "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00"
        },
        "onMenuOptionChanged_dayStartTimeSpring"
    )
		self.boxLayout:addElement(self.menuOption_dayStartTimeSpring)

    -- Add button for dayEndTimeSpring 
    self.menuOption_dayEndTimeSpring = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayEndTimeSpring",
        "setting_realTimeSync_dayEndTimeSpring",
        "toolTip_realTimeSync_dayEndTimeSpring",
        {
            "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00"
        },
        "onMenuOptionChanged_dayEndTimeSpring"
    )
		self.boxLayout:addElement(self.menuOption_dayEndTimeSpring)

    -- Add button for dayStartTimeSummer 
    self.menuOption_dayStartTimeSummer = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayStartTimeSummer",
        "setting_realTimeSync_dayStartTimeSummer",
        "toolTip_realTimeSync_dayStartTimeSummer",
        {
            "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00"
        },
        "onMenuOptionChanged_dayStartTimeSummer"
    )
		self.boxLayout:addElement(self.menuOption_dayStartTimeSummer)

    -- Add button for dayEndTimeSummer 
    self.menuOption_dayEndTimeSummer = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayEndTimeSummer",
        "setting_realTimeSync_dayEndTimeSummer",
        "toolTip_realTimeSync_dayEndTimeSummer",
        {
            "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00"
        },
        "onMenuOptionChanged_dayEndTimeSummer"
    )
		self.boxLayout:addElement(self.menuOption_dayEndTimeSummer)

    -- Add button for dayStartTimeAutumn 
    self.menuOption_dayStartTimeAutumn = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayStartTimeAutumn",
        "setting_realTimeSync_dayStartTimeAutumn",
        "toolTip_realTimeSync_dayStartTimeAutumn",
        {
            "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00"
        },
        "onMenuOptionChanged_dayStartTimeAutumn"
    )
		self.boxLayout:addElement(self.menuOption_dayStartTimeAutumn)

    -- Add button for dayEndTimeAutumn 
    self.menuOption_dayEndTimeAutumn = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayEndTimeAutumn",
        "setting_realTimeSync_dayEndTimeAutumn",
        "toolTip_realTimeSync_dayEndTimeAutumn",
        {
            "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00"
        },
        "onMenuOptionChanged_dayEndTimeAutumn"
    )
		self.boxLayout:addElement(self.menuOption_dayEndTimeAutumn)

    -- Add button for dayStartTimeWinter 
    self.menuOption_dayStartTimeWinter = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayStartTimeWinter",
        "setting_realTimeSync_dayStartTimeWinter",
        "toolTip_realTimeSync_dayStartTimeWinter",
        {
            "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00"
        },
        "onMenuOptionChanged_dayStartTimeWinter"
    )
		self.boxLayout:addElement(self.menuOption_dayStartTimeWinter)

    -- Add button for dayEndTimeWinter 
    self.menuOption_dayEndTimeWinter = RealTimeSync.addMenuOption(
        self.checkAutoMotorStart,
        g_realTimeSync,
        "realTimeSync_dayEndTimeWinter",
        "setting_realTimeSync_dayEndTimeWinter",
        "toolTip_realTimeSync_dayEndTimeWinter",
        {
            "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00"
        },
        "onMenuOptionChanged_dayEndTimeWinter"
    )
		self.boxLayout:addElement(self.menuOption_dayEndTimeWinter)

		local textSizeTexts = {}
		for _, size in ipairs(g_realTimeSync.menuTextSizes) do
			table.insert(textSizeTexts, tostring(size) .. " px")
		end

	end

	-- Set Current Values
  if not self.isServer then
    -- If server, just get the data local
    self.menuOption_ServerOffset:setState(g_realTimeSync.settings:getValue("serverOffset"))
    self.menuOption_TimeFixHour:setState(g_realTimeSync.settings:getValue("timeFixHour"))
    self.menuOption_enableTimeSync:setIsChecked(g_realTimeSync.settings:getValue("enableTimeSync"))
    self.menuOption_autoSetTime:setIsChecked(g_realTimeSync.settings:getValue("autoSetTime"))
    self.menuOption_dayStartTimeSpring:setState(g_realTimeSync.settings:getValue("dayStartTimeSpring"))
    self.menuOption_dayEndTimeSpring:setState(g_realTimeSync.settings:getValue("dayEndTimeSpring"))
    self.menuOption_dayStartTimeSummer:setState(g_realTimeSync.settings:getValue("dayStartTimeSummer"))
    self.menuOption_dayEndTimeSummer:setState(g_realTimeSync.settings:getValue("dayEndTimeSummer"))
    self.menuOption_dayStartTimeAutumn:setState(g_realTimeSync.settings:getValue("dayStartTimeAutumn"))
    self.menuOption_dayEndTimeAutumn:setState(g_realTimeSync.settings:getValue("dayEndTimeAutumn"))
    self.menuOption_dayStartTimeWinter:setState(g_realTimeSync.settings:getValue("dayStartTimeWinter"))
    self.menuOption_dayEndTimeWinter:setState(g_realTimeSync.settings:getValue("dayEndTimeWinter"))
  else
    -- If client, get data from server

  end
end
function RealTimeSync:onMenuOptionChanged_serverOffset(state)
    self.settings:setValue("serverOffset", state)
    self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_boolOpt:serverOffset: %s', state)
    SettingEvent.sendEvent(5, "serverOffset", state)
  end
end

function RealTimeSync:onMenuOptionChanged_timeFixHour(state)
  self.settings:setValue("timeFixHour", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_boolOpt:timeFixHour: %s', state)
    SettingEvent.sendEvent(5, "timeFixHour", state)
  end
end

function RealTimeSync:onMenuOptionChanged_onOffOpt(state, info)
  self.settings:setValue(
    "enableTimeSync",
    state == CheckedOptionElement.STATE_CHECKED
  )
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    if state == 1 then state = false else state = true end
    --printf('RealTimeSync:onMenuOptionChanged_onOffOpt:enableTimeSync: %s', state)
    SettingEvent.sendEvent(2, "enableTimeSync", state)
  end
end

function RealTimeSync:onMenuOptionChanged_boolOpt(state, info)
  self.settings:setValue(
    "autoSetTime",
    state == CheckedOptionElement.STATE_CHECKED
  )
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    if state == 1 then state = false else state = true end
    --printf('RealTimeSync:onMenuOptionChanged_boolOpt:autoSetTime: %s', state)
    SettingEvent.sendEvent(2, "autoSetTime", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayStartTimeSpring(state)
  self.settings:setValue("dayStartTimeSpring", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayStartTimeSpring:dayStartTimeSpring: %s', state)
    SettingEvent.sendEvent(5, "dayStartTimeSpring", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayEndTimeSpring(state)
  self.settings:setValue("dayEndTimeSpring", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayEndTimeSpring:dayEndTimeSpring: %s', state)
    SettingEvent.sendEvent(5, "dayEndTimeSpring", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayStartTimeSummer(state)
  self.settings:setValue("dayStartTimeSummer", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayStartTimeSummer:dayStartTimeSummer: %s', state)
    SettingEvent.sendEvent(5, "dayStartTimeSummer", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayEndTimeSummer(state)
  self.settings:setValue("dayEndTimeSummer", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayEndTimeSummer:dayEndTimeSummer: %s', state)
    SettingEvent.sendEvent(5, "dayEndTimeSummer", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayStartTimeAutumn(state)
  self.settings:setValue("dayStartTimeAutumn", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayStartTimeAutumn:dayStartTimeAutumn: %s', state)
    SettingEvent.sendEvent(5, "dayStartTimeAutumn", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayEndTimeAutumn(state)
  self.settings:setValue("dayEndTimeAutumn", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayEndTimeAutumn:dayEndTimeAutumn: %s', state)
    SettingEvent.sendEvent(5, "dayEndTimeAutumn", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayStartTimeWinter(state)
  self.settings:setValue("dayStartTimeWinter", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayStartTimeWinter:dayStartTimeWinter: %s', state)
    SettingEvent.sendEvent(5, "dayStartTimeWinter", state)
  end
end

function RealTimeSync:onMenuOptionChanged_dayEndTimeWinter(state)
  self.settings:setValue("dayEndTimeWinter", state)
  self.settings:saveSettings()
  if not self.isServer then
    -- Send setting data to the server
    --printf('RealTimeSync:onMenuOptionChanged_dayEndTimeWinter:dayEndTimeWinter: %s', state)
    SettingEvent.sendEvent(5, "dayEndTimeWinter", state)
  end
end

function RealTimeSync:loadMap(name)
	self.timeHour = "%H";
	self.timeMinute = "%M";
	self.timeSecond = "%S";
  if RealTimeSync:singleServerCheck() then 
    -- Set the time speed to 1
    if g_currentMission.missionInfo.timeScale ~= 1 then	
      g_currentMission.missionInfo.timeScale = 1
    end
  end
  if self.isServer then
    self.settings:loadSettings()
  end
  -- Need to load settings from server here
end

function RealTimeSync:update(dt)
  -- Set Defaults
  local TimeDiff = 0
  local TimeSpeed = 1
  -- check if within the loopCount, and if server or single player.  Don't run on server clients.
  if g_updateLoopIndex % self.setValueTimerFrequency == 0 and RealTimeSync:singleServerCheck() then
    self.settings:loadSettings()
    if self.settings:getValue("enableTimeSync") == false then
      -- Check to see if just set as false.  if so then set speed to 1
      if self.justSetFalse == true then
        if RealTimeSync:isSinglePlayer() then
          g_currentMission.missionInfo.timeScale = 1
        else
          g_currentMission:setTimeScale(1);
        end
        self.justSetFalse = false
      end
      return
    else
      self.justSetFalse = true
    end
    -- Get server hour and min
    local getServerHour = getTime()
    local getServerMinute = getTime()
    -- Get current hour from time
    local getTotalHours = (getServerHour / 60) / 60
    local getTotalDays = (getTotalHours / 24) - math.floor(getTotalHours / 24)
    local ServerHour = math.floor(getTotalDays * 24)
    -- Get current minute from time
    local ServerMinuteUTC = ((getTotalDays * 24) - math.floor(getTotalDays * 24)) * 60
    local ServerMinCleanUTC = math.floor(ServerMinuteUTC)
    -- Setup the times so they work with game times
    local ServerMin = MathUtil.round(ServerMinuteUTC / 60, 2)
    local ServerTime = ServerHour + ServerMin
    -- Set defaults for later changed vars
    local AlwaysRun = false
    local SetServerTime = 0
    -- Convert timeFixHour to a read able format
    local setEnableTimeSync = self.settings:getValue("enableTimeSync")
    local setTimeFixHour = self.settings:getValue("timeFixHour")
    local setServerOffset = self.settings:getValue("serverOffset") - 11
    local setAutoSetTime = self.settings:getValue("autoSetTime")
    -- Start the time fix process if enabled
    local runTimeFixHour = 1
    if setTimeFixHour == 1 then
      AlwaysRun = true;
    else
      runTimeFixHour = setTimeFixHour - 2
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "setTimeFixHour: " .. runTimeFixHour)
    end
    -- get the current hour with offset if one is set
    -- check if there is a time offset - if so then offset the server time so we can match the game time to that offset
    if type(setServerOffset) == "number" and setServerOffset ~= 0 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:serverTimeBeforeOffset: " .. ServerHour + ServerMin)
      ServerHour = (ServerHour + setServerOffset);
    end
    -- check if timeFixHour is set and if so then check if that time matches current time.  Stop script if not 0 or hour matches
    if AlwaysRun == false and runTimeFixHour ~= tonumber(ServerHour) then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:TimeFixHour not Always and ServerHour does not match TimeFixHour: " .. ServerHour .. "~=" .. runTimeFixHour)
      return
    end
    -- Get the game time for comparison 
    local GameHour = g_currentMission.environment.currentHour
    local GameMinute = g_currentMission.environment.currentMinute
    local GameMin = MathUtil.round(GameMinute / 60, 2)
    local GameTime = GameHour + GameMin
    -- Set the current server time with minutes added
    ServerTime = ServerHour + ServerMin;
    -- Check if ServerTime is greater than 24 and subtract if so so that it is not getting an invalid number
    if ServerTime > 24 then
      ServerTime = (ServerTime - 24)
    elseif ServerTime < 0 then
      ServerTime = (ServerTime + 24)
    end
    -- Time to set the game to if game is too far ahead of server time
    local SetServerTime = ServerTime - 0.03

    -- get the time differance between server time and game time so we can see if we need to speed up or slow down
    TimeDiff = MathUtil.round(ServerTime - GameTime, 2);

    self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:SetServerTime: " .. SetServerTime)
    self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:ServerTime: " .. ServerTime)
    self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime: " .. GameTime)
    self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:TimeDiff: " .. TimeDiff)

    -- Set server time to global for notfication use
    self.ServerHour = ServerHour
    self.ServerMinCleanUTC = ServerMinCleanUTC

    -- Check if time if server time is greater than game time, if so then speed up time to catch up
    -- If server time is less than in game time, then pause time until caught up
    -- Need a 5 minute window to keep it from constantly changing

    -- Check if Game time is less than server time by more than 1 hour.  Speed time way up if so.
    if GameTime < ServerTime and TimeDiff > 1 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime < ServerTime and time is off by more than an hour - speed up time x240")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 240
      else
        g_currentMission:setTimeScale(240);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 240 -- Time scale speed


    -- Check if Game time is less than server time by more than 0.75 hours.  Speed time way up if so.
    elseif GameTime < ServerTime and TimeDiff > 0.75 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime < ServerTime and time is off by more than an hour - speed up time x120")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 120
      else
        g_currentMission:setTimeScale(120);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 120 -- Time scale speed

    -- Check if Game time is less than server time by more than 0.5 hours.  Speed time way up if so.
    elseif GameTime < ServerTime and TimeDiff > 0.5 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime < ServerTime and time is off by more than an hour - speed up time x60")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 60
      else
        g_currentMission:setTimeScale(60);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 60 -- Time scale speed

    -- Check if Game time is less than server time by more than 0.25 hours.  Speed time way up if so.
    elseif GameTime < ServerTime and TimeDiff > 0.25 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime < ServerTime and time is off by more than an hour - speed up time x30")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 30
      else
        g_currentMission:setTimeScale(30);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 30 -- Time scale speed

    -- Check if Game time is less than server time.  Speed time up if so.
    elseif GameTime < ServerTime and TimeDiff > 0 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime < ServerTime - speed up time x5")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 5
      else
        g_currentMission:setTimeScale(5);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 5 -- Time scale speed

    -- Check if Game time is greater than server time and set time is true.  Set the time to match
    elseif GameTime > ServerTime and TimeDiff < 0 and TimeDiff < -0.8 and setAutoSetTime == true then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime > ServerTime - slow way down time")
      -- Looks to be changing both server time and game time
      g_currentMission.environment:consoleCommandSetDayTime(SetServerTime);
      -- Check if server then broadcast, if not then send local
      local setNotificationData = g_i18n:getText("title_realTimeSync_setInProgress") .. self.ServerHour .. ":" .. self.ServerMinCleanUTC - 2
      if self.isServer then
        g_server:broadcastEvent(SettingEvent.new(1, "realTimeSyncNoti", setNotificationData), nil, connection)
      else
        g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, setNotificationData)
      end
      self.ServerUpdate = false -- Enable notifcation
      TimeSpeed = 1 -- Time scale speed

    -- Check if Game time is greater than server time.  Slow time down if so.
    elseif GameTime > ServerTime and TimeDiff < -0.03 then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime > ServerTime - slow way down time")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 0.1
      else
        g_currentMission:setTimeScale(0.1);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 0.1 -- Time scale speed

    -- Check if Game time is greater than server time.  Slow time down if so.
    elseif GameTime > ServerTime then
      self.logger:print(":udpate()", FS22Log.LOG_LEVEL.VERBOSE, "RealTimeSync:GameTime > ServerTime - slow down time")
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 0.5
      else
        g_currentMission:setTimeScale(0.5);
      end
      self.ServerUpdate = true -- Enable notifcation
      TimeSpeed = 0.5 -- Time scale speed

    -- If time matches, then keep at scale 1
    elseif g_currentMission.missionInfo.timeScale ~= 1 then	  
      if RealTimeSync:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale = 1
      else
        g_currentMission:setTimeScale(1);
      end
      self.ServerUpdate = false -- Disable notifcation
      TimeSpeed = 1 -- Time scale speed
    end
  end
  -- Check if we should notify players that there was a change
  if self.ServerUpdate == true and TimeDiff ~= 0 and TimeSpeed ~= self.TimeSpeed then
    -- Add zerof to min if less than 10
    local addZero = "";
    if self.ServerMinCleanUTC < 10 then addZero = "0" else addZero = "" end
    -- Notify players time sync in progress
    local notificationData = g_i18n:getText("title_realTimeSync_inProgress") .. self.ServerHour .. ":" .. addZero .. self.ServerMinCleanUTC
    -- Check if server then broadcast, if not then send local
    if self.isServer then
      g_server:broadcastEvent(SettingEvent.new(1, "realTimeSyncNoti", notificationData), nil, connection)
    else
      g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, notificationData)
    end
    self.TimeSpeed = TimeSpeed -- Time scale speed
    self.ServerUpdate = false
  end
end

-- Disable the sleep stuffs
function RealTimeSync:disableSleep()
  return false
end

-- Check if client is a server or single player
function RealTimeSync:singleServerCheck()
  -- Check if server
  if g_currentMission:getIsServer() == true and g_currentMission:getIsClient() == true and g_dedicatedServer ~= nil and g_server ~= nil then 
    -- Multiplayer Server Host
    return true
  -- Check if single player
  elseif g_currentMission:getIsServer() == true and g_currentMission:getIsClient() == true and g_dedicatedServer == nil and g_server ~= nil then 
    -- Single player client
    return true
  else
    -- Client on dedicated server
    return false
  end
end

-- Check if single player
function RealTimeSync:isSinglePlayer()
  if g_currentMission:getIsServer() == true and g_currentMission:getIsClient() == true and g_dedicatedServer == nil and g_server ~= nil then 
    return true
  else
    return false
  end
end

-- Event when player joins server
function RealTimeSync:join()
  --print("RealTimeSync:join")
  -- Send settings to new client
  g_server:broadcastEvent(SettingEvent.new(5, "serverOffset", g_realTimeSync.settings:getValue("serverOffset")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "timeFixHour", g_realTimeSync.settings:getValue("timeFixHour")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(2, "enableTimeSync", g_realTimeSync.settings:getValue("enableTimeSync")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(2, "autoSetTime", g_realTimeSync.settings:getValue("autoSetTime")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayStartTimeSpring", g_realTimeSync.settings:getValue("dayStartTimeSpring")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayEndTimeSpring", g_realTimeSync.settings:getValue("dayEndTimeSpring")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayStartTimeSummer", g_realTimeSync.settings:getValue("dayStartTimeSummer")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayEndTimeSummer", g_realTimeSync.settings:getValue("dayEndTimeSummer")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayStartTimeAutumn", g_realTimeSync.settings:getValue("dayStartTimeAutumn")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayEndTimeAutumn", g_realTimeSync.settings:getValue("dayEndTimeAutumn")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayStartTimeWinter", g_realTimeSync.settings:getValue("dayStartTimeWinter")), nil, connection)
  g_server:broadcastEvent(SettingEvent.new(5, "dayEndTimeWinter", g_realTimeSync.settings:getValue("dayEndTimeWinter")), nil, connection)
end

function RealTimeSync:updateDayLightHours(self)
  --print('function:updateDayLightHours')
  local dayStartTime = 8
  local dayEndTime = 18

  local currentSeason = g_currentMission.environment.currentSeason
  --printf('RealTimeSync:updateDayLightHours:currentSeason: %s', currentSeason)

  -- Check to see which season the game is currently in, and get the times for that season
  if currentSeason == 1 then
    -- Get times for summer
    local dayStartTimeSummer = tonumber(g_realTimeSync.settings:getValue("dayStartTimeSummer") + 2)
    local dayEndTimeSummer = tonumber(g_realTimeSync.settings:getValue("dayEndTimeSummer") + 12)
    local seasonOutput = "Summer"
    -- Set times for summer, if missing use default
    if dayStartTimeSummer ~= nil then dayStartTime = tonumber(dayStartTimeSummer) end
    if dayEndTimeSummer ~= nil then dayEndTime = tonumber(dayEndTimeSummer) end 
  elseif currentSeason == 2 then 
    -- Get times for autumn
    local dayStartTimeAutumn = tonumber(g_realTimeSync.settings:getValue("dayStartTimeAutumn") + 2)
    local dayEndTimeAutumn = tonumber(g_realTimeSync.settings:getValue("dayEndTimeAutumn") + 12)
    local seasonOutput = "Autumn"
    -- Set times for autumn, if missing use default
    if dayStartTimeAutumn ~= nil then dayStartTime = tonumber(dayStartTimeAutumn) end
    if dayEndTimeAutumn ~= nil then dayEndTime = tonumber(dayEndTimeAutumn) end 
  elseif currentSeason == 3 then 
    -- Get times for winter
    local dayStartTimeWinter = tonumber(g_realTimeSync.settings:getValue("dayStartTimeWinter") + 2)
    local dayEndTimeWinter = tonumber(g_realTimeSync.settings:getValue("dayEndTimeWinter") + 12)
    local seasonOutput = "Winter"
    -- Set times for winter, if missing use default
    if dayStartTimeWinter ~= nil then dayStartTime = tonumber(dayStartTimeWinter) end
    if dayEndTimeWinter ~= nil then dayEndTime = tonumber(dayEndTimeWinter) end 
  else
    -- Get times for spring
    local dayStartTimeSpring = tonumber(g_realTimeSync.settings:getValue("dayStartTimeSpring") + 2)
    local dayEndTimeSpring = tonumber(g_realTimeSync.settings:getValue("dayEndTimeSpring") + 12)
    local seasonOutput = "Spring"
    -- Set times for spring, if missing use default
    if dayStartTimeSpring ~= nil then dayStartTime = tonumber(dayStartTimeSpring) end
    if dayEndTimeSpring ~= nil then dayEndTime = tonumber(dayEndTimeSpring) end 
  end

  --printf('RealTimeSync:updateDayLightHours:dayStartTime: %s', dayStartTime)
  --printf('RealTimeSync:updateDayLightHours:dayEndTime: %s', dayEndTime)

  -- Set the daylight settings
  g_currentMission.environment.daylight.dayStart = dayStartTime - 2            -- 8  = 6
	g_currentMission.environment.daylight.dayEnd = dayEndTime + 2                -- 18 = 20 
	g_currentMission.environment.daylight.nightEnd = dayStartTime                -- 8
	g_currentMission.environment.daylight.nightStart = dayEndTime                -- 18
	g_currentMission.environment.daylight.logicalNightStart = dayStartTime - 2   -- 8  = 6
	g_currentMission.environment.daylight.logicalNightEnd = dayEndTime           -- 18
  -- set the light settings
	g_currentMission.environment.lighting.dayStart = (dayStartTime - 2) + 0.55   -- 8  = 6.55
	g_currentMission.environment.lighting.dayEnd = dayEndTime + 0.1              -- 18 = 18.1
	g_currentMission.environment.lighting.nightEnd = dayStartTime - 3            -- 8  = 5
	g_currentMission.environment.lighting.nightStart = dayEndTime + 1.81         -- 18 = 19.81
end

function RealTimeSync:getDaylightTimes()
  --print('RealTimeSync:getDaylightTimes')
  local dayStartTime = 8
  local dayEndTime = 18
  local season = "Unknown"

  local currentSeason = g_currentMission.environment.currentSeason
  --printf('RealTimeSync:updateDayLightHours:currentSeason: %s', currentSeason)

  -- Check to see which season the game is currently in, and get the times for that season
  if currentSeason == 1 then
    -- Get times for summer
    local dayStartTimeSummer = tonumber(g_realTimeSync.settings:getValue("dayStartTimeSummer") + 2)
    local dayEndTimeSummer = tonumber(g_realTimeSync.settings:getValue("dayEndTimeSummer") + 12)
    seasonOutput = "Summer"
    -- Set times for summer, if missing use default
    if dayStartTimeSummer ~= nil then dayStartTime = tonumber(dayStartTimeSummer) end
    if dayEndTimeSummer ~= nil then dayEndTime = tonumber(dayEndTimeSummer) end 
  elseif currentSeason == 2 then 
    -- Get times for autumn
    local dayStartTimeAutumn = tonumber(g_realTimeSync.settings:getValue("dayStartTimeAutumn") + 2)
    local dayEndTimeAutumn = tonumber(g_realTimeSync.settings:getValue("dayEndTimeAutumn") + 12)
    seasonOutput = "Autumn"
    -- Set times for autumn, if missing use default
    if dayStartTimeAutumn ~= nil then dayStartTime = tonumber(dayStartTimeAutumn) end
    if dayEndTimeAutumn ~= nil then dayEndTime = tonumber(dayEndTimeAutumn) end 
  elseif currentSeason == 3 then 
    -- Get times for winter
    local dayStartTimeWinter = tonumber(g_realTimeSync.settings:getValue("dayStartTimeWinter") + 2)
    local dayEndTimeWinter = tonumber(g_realTimeSync.settings:getValue("dayEndTimeWinter") + 12)
    seasonOutput = "Winter"
    -- Set times for winter, if missing use default
    if dayStartTimeWinter ~= nil then dayStartTime = tonumber(dayStartTimeWinter) end
    if dayEndTimeWinter ~= nil then dayEndTime = tonumber(dayEndTimeWinter) end 
  else
    -- Get times for spring
    local dayStartTimeSpring = tonumber(g_realTimeSync.settings:getValue("dayStartTimeSpring") + 2)
    local dayEndTimeSpring = tonumber(g_realTimeSync.settings:getValue("dayEndTimeSpring") + 12)
    seasonOutput = "Spring"
    -- Set times for spring, if missing use default
    if dayStartTimeSpring ~= nil then dayStartTime = tonumber(dayStartTimeSpring) end
    if dayEndTimeSpring ~= nil then dayEndTime = tonumber(dayEndTimeSpring) end 
  end

  --printf('RealTimeSync:getDaylightTimes:dayStartTime: %s', dayStartTime)
  --printf('RealTimeSync:getDaylightTimes:dayEndTime: %s', dayEndTime)
  --printf('RealTimeSync:getDaylightTimes:seasonOutput: %s', seasonOutput)

  local dayStart = dayStartTime - 2
	local dayEnd = dayEndTime + 2
	local nightEnd = dayStartTime
	local nightStart = dayEndTime
	return dayStart, dayEnd, nightEnd, nightStart
end

Daylight.getDaylightTimes = Utils.overwrittenFunction(Daylight.getDaylightTimes, RealTimeSync.getDaylightTimes)

function RealTimeSync:setDaylightTimes()
  --print('RealTimeSync:setDaylightTimes')
  local dayStartTime = 8
  local dayEndTime = 18

  local currentSeason = g_currentMission.environment.currentSeason
  --printf('RealTimeSync:updateDayLightHours:currentSeason: %s', currentSeason)

  -- Check to see which season the game is currently in, and get the times for that season
  if currentSeason == 1 then
    -- Get times for summer
    local dayStartTimeSummer = tonumber(g_realTimeSync.settings:getValue("dayStartTimeSummer") + 2)
    local dayEndTimeSummer = tonumber(g_realTimeSync.settings:getValue("dayEndTimeSummer") + 12)
    local seasonOutput = "Summer"
    -- Set times for summer, if missing use default
    if dayStartTimeSummer ~= nil then dayStartTime = tonumber(dayStartTimeSummer) end
    if dayEndTimeSummer ~= nil then dayEndTime = tonumber(dayEndTimeSummer) end 
  elseif currentSeason == 2 then 
    -- Get times for autumn
    local dayStartTimeAutumn = tonumber(g_realTimeSync.settings:getValue("dayStartTimeAutumn") + 2)
    local dayEndTimeAutumn = tonumber(g_realTimeSync.settings:getValue("dayEndTimeAutumn") + 12)
    local seasonOutput = "Autumn"
    -- Set times for autumn, if missing use default
    if dayStartTimeAutumn ~= nil then dayStartTime = tonumber(dayStartTimeAutumn) end
    if dayEndTimeAutumn ~= nil then dayEndTime = tonumber(dayEndTimeAutumn) end 
  elseif currentSeason == 3 then 
    -- Get times for winter
    local dayStartTimeWinter = tonumber(g_realTimeSync.settings:getValue("dayStartTimeWinter") + 2)
    local dayEndTimeWinter = tonumber(g_realTimeSync.settings:getValue("dayEndTimeWinter") + 12)
    local seasonOutput = "Winter"
    -- Set times for winter, if missing use default
    if dayStartTimeWinter ~= nil then dayStartTime = tonumber(dayStartTimeWinter) end
    if dayEndTimeWinter ~= nil then dayEndTime = tonumber(dayEndTimeWinter) end 
  else
    -- Get times for spring
    local dayStartTimeSpring = tonumber(g_realTimeSync.settings:getValue("dayStartTimeSpring") + 2)
    local dayEndTimeSpring = tonumber(g_realTimeSync.settings:getValue("dayEndTimeSpring") + 12)
    local seasonOutput = "Spring"
    -- Set times for spring, if missing use default
    if dayStartTimeSpring ~= nil then dayStartTime = tonumber(dayStartTimeSpring) end
    if dayEndTimeSpring ~= nil then dayEndTime = tonumber(dayEndTimeSpring) end 
  end

  --printf('RealTimeSync:setDaylightTimes:dayStartTime: %s', dayStartTime)
  --printf('RealTimeSync:setDaylightTimes:dayEndTime: %s', dayEndTime)

  local dayStart = dayStartTime - 1
	local dayEnd = dayEndTime + 1
	local nightEnd = dayStartTime - 2
	local nightStart = dayEndTime + 2
	if g_currentMission.environment.lighting.dayStart ~= dayStart and g_currentMission.environment.lighting.dayEnd ~= dayEnd and g_currentMission.environment.lighting.nightEnd ~= nightEnd and g_currentMission.environment.lighting.nightStart ~= nightStart then
		g_currentMission.environment.lighting.dayStart = dayStart
		g_currentMission.environment.lighting.dayEnd = dayEnd
		g_currentMission.environment.lighting.nightEnd = nightEnd
		g_currentMission.environment.lighting.nightStart = nightStart

		g_currentMission.environment.lighting:updateCurves()
	end
end

Lighting.setDaylightTimes = Utils.overwrittenFunction(Lighting.setDaylightTimes, RealTimeSync.setDaylightTimes)
